# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class RolesV3TestJSON(base.BaseIdentityV3AdminTest):
    """Test roles"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @classmethod
    def resource_setup(cls):
        super(RolesV3TestJSON, cls).resource_setup()
        cls.roles = list()
        for _ in range(3):
            role_name = data_utils.rand_name(name='role')
            role = cls.roles_client.create_role(name=role_name)['role']
            cls.addClassResourceCleanup(cls.roles_client.delete_role,
                                        role['id'])
            cls.roles.append(role)
        u_name = data_utils.rand_name('user')
        u_desc = '%s description' % u_name
        u_email = '%s@testmail.tm' % u_name
        cls.u_password = data_utils.rand_password()
        cls.domain = cls.create_domain()
        cls.project = cls.projects_client.create_project(
            data_utils.rand_name('project'),
            description=data_utils.rand_name('project-desc'),
            domain_id=cls.domain['id'])['project']
        cls.addClassResourceCleanup(cls.projects_client.delete_project,
                                    cls.project['id'])
        cls.group_body = cls.groups_client.create_group(
            name=data_utils.rand_name('Group'), project_id=cls.project['id'],
            domain_id=cls.domain['id'])['group']
        cls.addClassResourceCleanup(cls.groups_client.delete_group,
                                    cls.group_body['id'])
        cls.role = cls.roles_client.create_role(
            name=data_utils.rand_name('Role'))['role']
        cls.addClassResourceCleanup(cls.roles_client.delete_role,
                                    cls.role['id'])
        if not CONF.identity_feature_enabled.immutable_user_source:
            cls.user_body = cls.users_client.create_user(
                name=u_name,
                description=u_desc,
                email=u_email,
                password=cls.u_password,
                domain_id=cls.domain['id'],
                project_id=cls.project['id']
            )['user']
            cls.addClassResourceCleanup(cls.users_client.delete_user,
                                        cls.user_body['id'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('18afc6c0-46cf-4911-824e-9989cc056c3a')
    def test_role_create_update_show_list(self):
        """Test creating, updating, showing and listing a role"""
        r_name = data_utils.rand_name('Role')
        role = self.roles_client.create_role(name=r_name)['role']
        self.addCleanup(self.roles_client.delete_role, role['id'])
        self.assertIn('name', role)
        self.assertEqual(role['name'], r_name)

        new_name = data_utils.rand_name('NewRole')
        updated_role = self.roles_client.update_role(role['id'],
                                                     name=new_name)['role']
        self.assertIn('name', updated_role)
        self.assertIn('id', updated_role)
        self.assertIn('links', updated_role)
        self.assertNotEqual(r_name, updated_role['name'])

        new_role = self.roles_client.show_role(role['id'])['role']
        self.assertEqual(new_name, new_role['name'])
        self.assertEqual(updated_role['id'], new_role['id'])

        roles = self.roles_client.list_roles()['roles']
        self.assertIn(role['id'], [r['id'] for r in roles])

    @decorators.idempotent_id('c6b80012-fe4a-498b-9ce8-eb391c05169f')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_grant_list_revoke_role_to_user_on_project(self):
        """Test granting, listing, revoking role to user on project"""
        self.roles_client.create_user_role_on_project(self.project['id'],
                                                      self.user_body['id'],
                                                      self.role['id'])

        roles = self.roles_client.list_user_roles_on_project(
            self.project['id'], self.user_body['id'])['roles']

        self.assertEqual(1, len(roles))
        self.assertEqual(self.role['id'], roles[0]['id'])

        self.roles_client.check_user_role_existence_on_project(
            self.project['id'], self.user_body['id'], self.role['id'])

        self.roles_client.delete_role_from_user_on_project(
            self.project['id'], self.user_body['id'], self.role['id'])

    @decorators.idempotent_id('6c9a2940-3625-43a3-ac02-5dcec62ef3bd')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_grant_list_revoke_role_to_user_on_domain(self):
        """Test granting, listing, revoking role to user on domain"""
        self.roles_client.create_user_role_on_domain(
            self.domain['id'], self.user_body['id'], self.role['id'])

        roles = self.roles_client.list_user_roles_on_domain(
            self.domain['id'], self.user_body['id'])['roles']

        self.assertEqual(1, len(roles))
        self.assertEqual(self.role['id'], roles[0]['id'])

        self.roles_client.check_user_role_existence_on_domain(
            self.domain['id'], self.user_body['id'], self.role['id'])

        self.roles_client.delete_role_from_user_on_domain(
            self.domain['id'], self.user_body['id'], self.role['id'])

    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    @decorators.idempotent_id('e5a81737-d294-424d-8189-8664858aae4c')
    def test_grant_list_revoke_role_to_user_on_system(self):
        self.roles_client.create_user_role_on_system(
            self.user_body['id'], self.role['id'])

        roles = self.roles_client.list_user_roles_on_system(
            self.user_body['id'])['roles']

        self.assertEqual(1, len(roles))
        self.assertEqual(self.role['id'], roles[0]['id'])

        self.roles_client.check_user_role_existence_on_system(
            self.user_body['id'], self.role['id'])

        self.roles_client.delete_role_from_user_on_system(
            self.user_body['id'], self.role['id'])

    @decorators.idempotent_id('cbf11737-1904-4690-9613-97bcbb3df1c4')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_grant_list_revoke_role_to_group_on_project(self):
        """Test granting, listing, revoking role to group on project"""
        # Grant role to group on project
        self.roles_client.create_group_role_on_project(
            self.project['id'], self.group_body['id'], self.role['id'])
        # List group roles on project
        roles = self.roles_client.list_group_roles_on_project(
            self.project['id'], self.group_body['id'])['roles']

        self.assertEqual(1, len(roles))
        self.assertEqual(self.role['id'], roles[0]['id'])

        # Add user to group, and insure user has role on project
        self.groups_client.add_group_user(self.group_body['id'],
                                          self.user_body['id'])
        self.addCleanup(self.groups_client.delete_group_user,
                        self.group_body['id'], self.user_body['id'])
        body = self.token.auth(user_id=self.user_body['id'],
                               password=self.u_password,
                               user_domain_name=self.domain['name'],
                               project_name=self.project['name'],
                               project_domain_name=self.domain['name'])
        roles = body['token']['roles']
        self.assertEqual(len(roles), 1)
        self.assertEqual(roles[0]['id'], self.role['id'])

        self.roles_client.check_role_from_group_on_project_existence(
            self.project['id'], self.group_body['id'], self.role['id'])

        # Revoke role to group on project
        self.roles_client.delete_role_from_group_on_project(
            self.project['id'], self.group_body['id'], self.role['id'])

    @decorators.idempotent_id('4bf8a70b-e785-413a-ad53-9f91ce02faa7')
    def test_grant_list_revoke_role_to_group_on_domain(self):
        """Test granting, listing, revoking role to group on domain"""
        self.roles_client.create_group_role_on_domain(
            self.domain['id'], self.group_body['id'], self.role['id'])

        roles = self.roles_client.list_group_roles_on_domain(
            self.domain['id'], self.group_body['id'])['roles']

        self.assertEqual(1, len(roles))
        self.assertEqual(self.role['id'], roles[0]['id'])

        self.roles_client.check_role_from_group_on_domain_existence(
            self.domain['id'], self.group_body['id'], self.role['id'])

        self.roles_client.delete_role_from_group_on_domain(
            self.domain['id'], self.group_body['id'], self.role['id'])

    @decorators.idempotent_id('c888fe4f-8018-48db-b959-542225c1b4b6')
    def test_grant_list_revoke_role_to_group_on_system(self):
        self.roles_client.create_group_role_on_system(
            self.group_body['id'], self.role['id'])

        roles = self.roles_client.list_group_roles_on_system(
            self.group_body['id'])['roles']

        self.assertEqual(1, len(roles))
        self.assertEqual(self.role['id'], roles[0]['id'])

        self.roles_client.check_role_from_group_on_system_existence(
            self.group_body['id'], self.role['id'])

        self.roles_client.delete_role_from_group_on_system(
            self.group_body['id'], self.role['id'])

    @decorators.idempotent_id('f5654bcc-08c4-4f71-88fe-05d64e06de94')
    def test_list_roles(self):
        """Test listing roles"""
        # Return a list of all roles
        body = self.roles_client.list_roles()['roles']
        found = [role for role in body if role in self.roles]
        self.assertEqual(len(found), len(self.roles))

    def _create_implied_role(self, prior_role_id, implies_role_id,
                             ignore_not_found=False):
        self.roles_client.create_role_inference_rule(
            prior_role_id, implies_role_id)
        if ignore_not_found:
            self.addCleanup(
                test_utils.call_and_ignore_notfound_exc,
                self.roles_client.delete_role_inference_rule,
                prior_role_id,
                implies_role_id)
        else:
            self.addCleanup(
                self.roles_client.delete_role_inference_rule,
                prior_role_id,
                implies_role_id)

    @decorators.idempotent_id('c90c316c-d706-4728-bcba-eb1912081b69')
    def test_implied_roles_create_check_show_delete(self):
        """Test creating, checking, showing and deleting implied roles"""
        prior_role_id = self.roles[0]['id']
        implies_role_id = self.roles[1]['id']

        # Create an inference rule from prior_role to implies_role
        self._create_implied_role(prior_role_id, implies_role_id,
                                  ignore_not_found=True)

        # Check if the inference rule exists
        self.roles_client.check_role_inference_rule(
            prior_role_id, implies_role_id)

        # Show the inference rule and check its elements
        resp_body = self.roles_client.show_role_inference_rule(
            prior_role_id, implies_role_id)
        self.assertIn('role_inference', resp_body)
        role_inference = resp_body['role_inference']
        for key1 in ['prior_role', 'implies']:
            self.assertIn(key1, role_inference)
            for key2 in ['id', 'links', 'name']:
                self.assertIn(key2, role_inference[key1])

        # Delete the inference rule
        self.roles_client.delete_role_inference_rule(
            prior_role_id, implies_role_id)
        # Check if the inference rule no longer exists
        self.assertRaises(
            lib_exc.NotFound,
            self.roles_client.show_role_inference_rule,
            prior_role_id,
            implies_role_id)

    @decorators.idempotent_id('dc6f5959-b74d-4e30-a9e5-a8255494ff00')
    def test_roles_hierarchy(self):
        """Test creating implied role and listing role inferences rules"""
        # Create inference rule from "roles[0]" to "role[1]"
        self._create_implied_role(
            self.roles[0]['id'], self.roles[1]['id'])

        # Create inference rule from "roles[0]" to "role[2]"
        self._create_implied_role(
            self.roles[0]['id'], self.roles[2]['id'])

        # Create inference rule from "roles[2]" to "role"
        self._create_implied_role(
            self.roles[2]['id'], self.role['id'])

        # Listing inferences rules from "roles[2]" should only return "role"
        rules = self.roles_client.list_role_inferences_rules(
            self.roles[2]['id'])['role_inference']
        self.assertEqual(1, len(rules['implies']))
        self.assertEqual(self.role['id'], rules['implies'][0]['id'])

        # Listing inferences rules from "roles[0]" should return "roles[1]" and
        # "roles[2]" (only direct rules are listed)
        rules = self.roles_client.list_role_inferences_rules(
            self.roles[0]['id'])['role_inference']
        implies_ids = [role['id'] for role in rules['implies']]
        self.assertEqual(2, len(implies_ids))
        self.assertIn(self.roles[1]['id'], implies_ids)
        self.assertIn(self.roles[2]['id'], implies_ids)

    @decorators.idempotent_id('c8828027-df48-4021-95df-b65b92c7429e')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_assignments_for_implied_roles_create_delete(self):
        """Test assignments when implied roles are created and deleted"""
        # Create a grant using "roles[0]"
        self.roles_client.create_user_role_on_project(
            self.project['id'], self.user_body['id'], self.roles[0]['id'])
        self.addCleanup(
            self.roles_client.delete_role_from_user_on_project,
            self.project['id'], self.user_body['id'], self.roles[0]['id'])

        # Create an inference rule from "roles[0]" to "roles[1]"
        self._create_implied_role(self.roles[0]['id'], self.roles[1]['id'],
                                  ignore_not_found=True)

        # In the effective list of role assignments, both prior role and
        # implied role should be present. This means that a user can
        # authenticate using both roles (both roles will be present
        # in the token).
        params = {'scope.project.id': self.project['id'],
                  'user.id': self.user_body['id']}
        role_assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertEqual(2, len(role_assignments))

        roles_ids = [assignment['role']['id']
                     for assignment in role_assignments]
        self.assertIn(self.roles[0]['id'], roles_ids)
        self.assertIn(self.roles[1]['id'], roles_ids)

        # After deleting the implied role, only the assignment with "roles[0]"
        # should be present.
        self.roles_client.delete_role_inference_rule(
            self.roles[0]['id'], self.roles[1]['id'])

        role_assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertEqual(1, len(role_assignments))

        roles_ids = [assignment['role']['id']
                     for assignment in role_assignments]
        self.assertIn(self.roles[0]['id'], roles_ids)

    @decorators.idempotent_id('d92a41d2-5501-497a-84bb-6e294330e8f8')
    def test_domain_roles_create_delete(self):
        """Test creating, listing and deleting domain roles"""
        domain_role = self.roles_client.create_role(
            name=data_utils.rand_name('domain_role'),
            domain_id=self.domain['id'])['role']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.roles_client.delete_role,
            domain_role['id'])

        domain_roles = self.roles_client.list_roles(
            domain_id=self.domain['id'])['roles']
        self.assertEqual(1, len(domain_roles))
        self.assertIn(domain_role, domain_roles)

        self.roles_client.delete_role(domain_role['id'])
        domain_roles = self.roles_client.list_roles(
            domain_id=self.domain['id'])['roles']
        self.assertEmpty(domain_roles)

    @decorators.idempotent_id('eb1e1c24-1bc4-4d47-9748-e127a1852c82')
    def test_implied_domain_roles(self):
        """Test creating implied roles when roles are in domains"""
        # Create two roles in the same domain
        domain_role1 = self.setup_test_role(domain_id=self.domain['id'])
        domain_role2 = self.setup_test_role(domain_id=self.domain['id'])

        # Check if we can create an inference rule from roles in the same
        # domain
        self._create_implied_role(domain_role1['id'], domain_role2['id'])

        # Create another role in a different domain
        domain2 = self.setup_test_domain()
        domain_role3 = self.setup_test_role(domain_id=domain2['id'])

        # Check if we can create cross domain implied roles
        self._create_implied_role(domain_role1['id'], domain_role3['id'])

        # Finally, we also should be able to create an implied from a
        # domain role to a global one
        self._create_implied_role(domain_role1['id'], self.role['id'])

        # The contrary is not true: we can't create an inference rule
        # from a global role to a domain role
        self.assertRaises(
            lib_exc.Forbidden,
            self.roles_client.create_role_inference_rule,
            self.role['id'],
            domain_role1['id'])

    @decorators.idempotent_id('3859df7e-5b78-4e4d-b10e-214c8953842a')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_assignments_for_domain_roles(self):
        """Test assignments for domain roles"""
        domain_role = self.setup_test_role(domain_id=self.domain['id'])

        # Create a grant using "domain_role"
        self.roles_client.create_user_role_on_project(
            self.project['id'], self.user_body['id'], domain_role['id'])
        self.addCleanup(
            self.roles_client.delete_role_from_user_on_project,
            self.project['id'], self.user_body['id'], domain_role['id'])

        # NOTE(rodrigods): Regular roles would appear in the effective
        # list of role assignments (meaning the role would be returned in
        # a token) as a result from the grant above. This is not the case
        # for domain roles, they should not appear in the effective role
        # assignments list.
        params = {'scope.project.id': self.project['id'],
                  'user.id': self.user_body['id']}
        role_assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertEmpty(role_assignments)

    @decorators.idempotent_id('3748c316-c18f-4b08-997b-c60567bc6235')
    def test_list_all_implied_roles(self):
        """Test listing all implied roles"""
        # Create inference rule from "roles[0]" to "roles[1]"
        self._create_implied_role(
            self.roles[0]['id'], self.roles[1]['id'])

        # Create inference rule from "roles[0]" to "roles[2]"
        self._create_implied_role(
            self.roles[0]['id'], self.roles[2]['id'])

        # Create inference rule from "roles[2]" to "role"
        self._create_implied_role(
            self.roles[2]['id'], self.role['id'])

        rules = self.roles_client.list_all_role_inference_rules()[
            'role_inferences']

        # NOTE(jaosorior): With the work related to the define-default-roles
        # blueprint, we now have 'admin', 'member' and 'reader' by default. So
        # we filter every other implied role to only take into account the ones
        # relates to this test.
        relevant_roles = (self.roles[0]['id'], self.roles[1]['id'],
                          self.roles[2]['id'], self.role['id'])

        def is_implied_role_relevant(rule):
            return any(r for r in rule['implies'] if r['id'] in relevant_roles)

        relevant_rules = filter(is_implied_role_relevant, rules)
        # Sort the rules by the number of inferences, since there should be 1
        # inference between "roles[2]" and "role" and 2 inferences for
        # "roles[0]": between "roles[1]" and "roles[2]".
        sorted_rules = sorted(relevant_rules, key=lambda r: len(r['implies']))

        self.assertEqual(2, len(sorted_rules))
        # Check that only 1 inference rule exists between "roles[2]" and "role"
        self.assertEqual(1, len(sorted_rules[0]['implies']))
        # Check that 2 inference rules exist for "roles[0]": one between
        # "roles[1]" and one between "roles[2]".
        self.assertEqual(2, len(sorted_rules[1]['implies']))

        # Check that "roles[2]" is the "prior_role" and that "role" is the
        # "implies" role.
        self.assertEqual(self.roles[2]['id'],
                         sorted_rules[0]['prior_role']['id'])
        self.assertEqual(self.role['id'],
                         sorted_rules[0]['implies'][0]['id'])

        # Check that "roles[0]" is the "prior_role" and that "roles[1]" and
        # "roles[2]" are the "implies" roles.
        self.assertEqual(self.roles[0]['id'],
                         sorted_rules[1]['prior_role']['id'])

        implies_ids = [r['id'] for r in sorted_rules[1]['implies']]
        self.assertIn(self.roles[1]['id'], implies_ids)
        self.assertIn(self.roles[2]['id'], implies_ids)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from tempest.api.image import base
from tempest.common import image as common_image
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


def get_container_and_disk_format():
    a_formats = ['ami', 'ari', 'aki']

    container_format = CONF.image.container_formats[0]

    # In v1, If container_format is one of ['ami', 'ari', 'aki'], then
    # disk_format must be same with container_format.
    # If they are of different item sequence in tempest.conf, such as:
    #     container_formats = ami,ari,aki,bare
    #     disk_formats = ari,ami,aki,vhd
    # we can select one in disk_format list that is same with container_format.
    if container_format in a_formats:
        if container_format in CONF.image.disk_formats:
            disk_format = container_format
        else:
            msg = ("The container format and the disk format don't match. "
                   "Container format: %(container)s, Disk format: %(disk)s." %
                   {'container': container_format, 'disk':
                       CONF.image.disk_formats})
            raise exceptions.InvalidConfiguration(msg)
    else:
        disk_format = CONF.image.disk_formats[0]

    return container_format, disk_format


class CreateRegisterImagesTest(base.BaseV1ImageTest):
    """Here we test the registration and creation of images."""

    @decorators.idempotent_id('3027f8e6-3492-4a11-8575-c3293017af4d')
    def test_register_then_upload(self):
        """Register, then upload an image"""
        properties = {'prop1': 'val1'}
        container_format, disk_format = get_container_and_disk_format()
        image = self.create_image(name='New Name',
                                  container_format=container_format,
                                  disk_format=disk_format,
                                  is_public=False,
                                  properties=properties)
        self.assertEqual('New Name', image.get('name'))
        self.assertFalse(image.get('is_public'))
        self.assertEqual('queued', image.get('status'))
        for key, val in properties.items():
            self.assertEqual(val, image.get('properties')[key])

        # Now try uploading an image file
        image_file = io.BytesIO(data_utils.random_bytes())
        body = self.client.update_image(image['id'], data=image_file)['image']
        self.assertIn('size', body)
        self.assertEqual(1024, body.get('size'))

    @decorators.idempotent_id('69da74d9-68a9-404b-9664-ff7164ccb0f5')
    def test_register_remote_image(self):
        """Register a new remote image"""
        container_format, disk_format = get_container_and_disk_format()
        body = self.create_image(name='New Remote Image',
                                 container_format=container_format,
                                 disk_format=disk_format, is_public=False,
                                 location=CONF.image.http_image,
                                 properties={'key1': 'value1',
                                             'key2': 'value2'})
        self.assertEqual('New Remote Image', body.get('name'))
        self.assertFalse(body.get('is_public'))
        self.assertEqual('active', body.get('status'))
        properties = body.get('properties')
        self.assertEqual(properties['key1'], 'value1')
        self.assertEqual(properties['key2'], 'value2')

    @decorators.idempotent_id('6d0e13a7-515b-460c-b91f-9f4793f09816')
    def test_register_http_image(self):
        """Register a new image from an http image path url"""
        container_format, disk_format = get_container_and_disk_format()
        image = self.create_image(name='New Http Image',
                                  container_format=container_format,
                                  disk_format=disk_format, is_public=False,
                                  copy_from=CONF.image.http_image)
        self.assertEqual('New Http Image', image.get('name'))
        self.assertFalse(image.get('is_public'))
        waiters.wait_for_image_status(self.client, image['id'], 'active')
        self.client.show_image(image['id'])

    @decorators.idempotent_id('05b19d55-140c-40d0-b36b-fafd774d421b')
    def test_register_image_with_min_ram(self):
        """Register an image with min ram"""
        container_format, disk_format = get_container_and_disk_format()
        properties = {'prop1': 'val1'}
        body = self.create_image(name='New_image_with_min_ram',
                                 container_format=container_format,
                                 disk_format=disk_format,
                                 is_public=False,
                                 min_ram=40,
                                 properties=properties)
        self.assertEqual('New_image_with_min_ram', body.get('name'))
        self.assertFalse(body.get('is_public'))
        self.assertEqual('queued', body.get('status'))
        self.assertEqual(40, body.get('min_ram'))
        for key, val in properties.items():
            self.assertEqual(val, body.get('properties')[key])
        self.client.delete_image(body['id'])


class ListImagesTest(base.BaseV1ImageTest):
    """Here we test the listing of image information"""

    @classmethod
    def skip_checks(cls):
        super(ListImagesTest, cls).skip_checks()
        if (len(CONF.image.container_formats) < 2 or
                len(CONF.image.disk_formats) < 2):
            skip_msg = ("%s skipped as multiple container formats "
                        "or disk formats are not available." % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(ListImagesTest, cls).resource_setup()
        # We add a few images here to test the listing functionality of
        # the images API
        a_formats = ['ami', 'ari', 'aki']

        (cls.container_format,
         container_format_alt) = CONF.image.container_formats[:2]
        cls.disk_format, cls.disk_format_alt = CONF.image.disk_formats[:2]
        if cls.container_format in a_formats:
            cls.disk_format = cls.container_format
        if container_format_alt in a_formats:
            cls.disk_format_alt = container_format_alt

        img1 = cls._create_remote_image('one', cls.container_format,
                                        cls.disk_format)
        img2 = cls._create_remote_image('two', container_format_alt,
                                        cls.disk_format_alt)
        img3 = cls._create_remote_image('dup', cls.container_format,
                                        cls.disk_format)
        img4 = cls._create_remote_image('dup', cls.container_format,
                                        cls.disk_format)
        img5 = cls._create_standard_image('1', container_format_alt,
                                          cls.disk_format_alt, 42)
        img6 = cls._create_standard_image('2', container_format_alt,
                                          cls.disk_format_alt, 142)
        img7 = cls._create_standard_image('33', cls.container_format,
                                          cls.disk_format, 142)
        img8 = cls._create_standard_image('33', cls.container_format,
                                          cls.disk_format, 142)
        cls.created_set = set(cls.created_images)
        # same container format
        cls.same_container_format_set = set((img1, img3, img4, img7, img8))
        # same disk format
        cls.same_disk_format_set = set((img2, img5, img6))

        # 1x with size 42
        cls.size42_set = set((img5,))
        # 3x with size 142
        cls.size142_set = set((img6, img7, img8,))
        # dup named
        cls.dup_set = set((img3, img4))

    @classmethod
    def _create_remote_image(cls, name, container_format, disk_format):
        """Create a new remote image and return newly-registered image-id"""

        name = 'New Remote Image %s' % name
        location = CONF.image.http_image
        image = cls.create_image(name=name,
                                 container_format=container_format,
                                 disk_format=disk_format,
                                 is_public=False,
                                 location=location)
        return image['id']

    @classmethod
    def _create_standard_image(cls, name, container_format,
                               disk_format, size):
        """Create a new standard image and return newly-registered image-id

        Note that the size of the new image is a random number between
        1024 and 4096
        """
        image_file = io.BytesIO(data_utils.random_bytes(size))
        name = 'New Standard Image %s' % name
        image = cls.create_image(name=name,
                                 container_format=container_format,
                                 disk_format=disk_format,
                                 is_public=False, data=image_file)
        return image['id']

    @decorators.idempotent_id('246178ab-3b33-4212-9a4b-a7fe8261794d')
    def test_index_no_params(self):
        """Simple test to see all fixture images returned"""
        images_list = self.client.list_images()['images']
        image_list = [image['id'] for image in images_list]
        for image_id in self.created_images:
            self.assertIn(image_id, image_list)

    @decorators.idempotent_id('f1755589-63d6-4468-b098-589820eb4031')
    def test_index_disk_format(self):
        """Test listing images by disk format"""
        images_list = self.client.list_images(
            disk_format=self.disk_format_alt)['images']
        for image in images_list:
            self.assertEqual(image['disk_format'], self.disk_format_alt)
        result_set = set(map(lambda x: x['id'], images_list))
        self.assertTrue(self.same_disk_format_set <= result_set)
        self.assertFalse(self.created_set - self.same_disk_format_set <=
                         result_set)

    @decorators.idempotent_id('2143655d-96d9-4bec-9188-8674206b4b3b')
    def test_index_container_format(self):
        """Test listing images by container format"""
        images_list = self.client.list_images(
            container_format=self.container_format)['images']
        for image in images_list:
            self.assertEqual(image['container_format'], self.container_format)
        result_set = set(map(lambda x: x['id'], images_list))
        self.assertTrue(self.same_container_format_set <= result_set)
        self.assertFalse(self.created_set - self.same_container_format_set <=
                         result_set)

    @decorators.idempotent_id('feb32ac6-22bb-4a16-afd8-9454bb714b14')
    def test_index_max_size(self):
        """Test listing images by max size"""
        images_list = self.client.list_images(size_max=42)['images']
        for image in images_list:
            self.assertLessEqual(image['size'], 42)
        result_set = set(map(lambda x: x['id'], images_list))
        self.assertTrue(self.size42_set <= result_set)
        self.assertFalse(self.created_set - self.size42_set <= result_set)

    @decorators.idempotent_id('6ffc16d0-4cbf-4401-95c8-4ac63eac34d8')
    def test_index_min_size(self):
        """Test listing images by min size"""
        images_list = self.client.list_images(size_min=142)['images']
        for image in images_list:
            self.assertGreaterEqual(image['size'], 142)
        result_set = set(map(lambda x: x['id'], images_list))
        self.assertTrue(self.size142_set <= result_set)
        self.assertFalse(self.size42_set <= result_set)

    @decorators.idempotent_id('e5dc26d9-9aa2-48dd-bda5-748e1445da98')
    def test_index_status_active_detail(self):
        """Test listing active images sorting by size in descending order"""
        images_list = self.client.list_images(detail=True,
                                              status='active',
                                              sort_key='size',
                                              sort_dir='desc')['images']
        top_size = images_list[0]['size']  # We have non-zero sized images
        for image in images_list:
            size = image['size']
            self.assertLessEqual(size, top_size)
            top_size = size
            self.assertEqual(image['status'], 'active')

    @decorators.idempotent_id('097af10a-bae8-4342-bff4-edf89969ed2a')
    def test_index_name(self):
        """Test listing images by its name"""
        images_list = self.client.list_images(
            detail=True,
            name='New Remote Image dup')['images']
        result_set = set(map(lambda x: x['id'], images_list))
        for image in images_list:
            self.assertEqual(image['name'], 'New Remote Image dup')
        self.assertTrue(self.dup_set <= result_set)
        self.assertFalse(self.created_set - self.dup_set <= result_set)


class UpdateImageMetaTest(base.BaseV1ImageTest):
    """Test image metadata"""

    @classmethod
    def resource_setup(cls):
        super(UpdateImageMetaTest, cls).resource_setup()
        container_format, disk_format = get_container_and_disk_format()
        cls.image_id = cls._create_standard_image('1', container_format,
                                                  disk_format, 42)

    @classmethod
    def _create_standard_image(cls, name, container_format,
                               disk_format, size):
        """Create a new standard image and return newly-registered image-id"""

        image_file = io.BytesIO(data_utils.random_bytes(size))
        name = 'New Standard Image %s' % name
        image = cls.create_image(name=name,
                                 container_format=container_format,
                                 disk_format=disk_format,
                                 is_public=False, data=image_file,
                                 properties={'key1': 'value1'})
        return image['id']

    @decorators.idempotent_id('01752c1c-0275-4de3-9e5b-876e44541928')
    def test_list_image_metadata(self):
        """Test listing image metadata"""
        # All metadata key/value pairs for an image should be returned
        resp = self.client.check_image(self.image_id)
        resp_metadata = common_image.get_image_meta_from_headers(resp)
        expected = {'key1': 'value1'}
        self.assertEqual(expected, resp_metadata['properties'])

    @decorators.idempotent_id('d6d7649c-08ce-440d-9ea7-e3dda552f33c')
    def test_update_image_metadata(self):
        """Test updating image metadata"""
        # The metadata for the image should match the updated values
        req_metadata = {'key1': 'alt1', 'key2': 'value2'}
        resp = self.client.check_image(self.image_id)
        metadata = common_image.get_image_meta_from_headers(resp)
        self.assertEqual(metadata['properties'], {'key1': 'value1'})
        metadata['properties'].update(req_metadata)
        headers = common_image.image_meta_to_headers(
            properties=metadata['properties'])
        self.client.update_image(self.image_id, headers=headers)
        resp = self.client.check_image(self.image_id)
        resp_metadata = common_image.get_image_meta_from_headers(resp)
        self.assertEqual(req_metadata, resp_metadata['properties'])



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class ServersOnMultiNodesTest(base.BaseV2ComputeAdminTest):
    """Test creating servers on mutiple nodes with scheduler_hints."""
    @classmethod
    def resource_setup(cls):
        super(ServersOnMultiNodesTest, cls).resource_setup()
        cls.server01 = cls.create_test_server(wait_until='ACTIVE')['id']
        cls.host01 = cls.get_host_for_server(cls.server01)

    @classmethod
    def skip_checks(cls):
        super(ServersOnMultiNodesTest, cls).skip_checks()

        if CONF.compute.min_compute_nodes < 2:
            raise cls.skipException(
                "Less than 2 compute nodes, skipping multi-nodes test.")

    def _create_servers_with_group(self, policy):
        group_id = self.create_test_server_group(policy=[policy])['id']
        hints = {'group': group_id}
        reservation_id = self.create_test_server(
            scheduler_hints=hints, wait_until='ACTIVE', min_count=2,
            return_reservation_id=True)['reservation_id']

        # Get the servers using the reservation_id.
        servers = self.servers_client.list_servers(
            detail=True, reservation_id=reservation_id)['servers']
        self.assertEqual(2, len(servers))

        # Assert the servers are in the group.
        server_group = self.server_groups_client.show_server_group(
            group_id)['server_group']
        hosts = {}
        for server in servers:
            self.assertIn(server['id'], server_group['members'])
            hosts[server['id']] = self.get_host_for_server(server['id'])

        return hosts

    @decorators.idempotent_id('26a9d5df-6890-45f2-abc4-a659290cb130')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("SameHostFilter"),
        'SameHostFilter is not available.')
    def test_create_servers_on_same_host(self):
        """Test creating servers with hints 'same_host'"""
        hints = {'same_host': self.server01}
        server02 = self.create_test_server(scheduler_hints=hints,
                                           wait_until='ACTIVE')['id']
        host02 = self.get_host_for_server(server02)
        self.assertEqual(self.host01, host02)

    @decorators.idempotent_id('cc7ca884-6e3e-42a3-a92f-c522fcf25e8e')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("DifferentHostFilter"),
        'DifferentHostFilter is not available.')
    def test_create_servers_on_different_hosts(self):
        """Test creating servers with hints of single 'different_host'"""
        hints = {'different_host': self.server01}
        server02 = self.create_test_server(scheduler_hints=hints,
                                           wait_until='ACTIVE')['id']
        host02 = self.get_host_for_server(server02)
        self.assertNotEqual(self.host01, host02)

    @decorators.idempotent_id('7869cc84-d661-4e14-9f00-c18cdc89cf57')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("DifferentHostFilter"),
        'DifferentHostFilter is not available.')
    def test_create_servers_on_different_hosts_with_list_of_servers(self):
        """Test creating servers with hints of a list of 'different_host'"""
        hints = {'different_host': [self.server01]}
        server02 = self.create_test_server(scheduler_hints=hints,
                                           wait_until='ACTIVE')['id']
        host02 = self.get_host_for_server(server02)
        self.assertNotEqual(self.host01, host02)

    @decorators.idempotent_id('f8bd0867-e459-45f5-ba53-59134552fe04')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAntiAffinityFilter"),
        'ServerGroupAntiAffinityFilter is not available.')
    def test_create_server_with_scheduler_hint_group_anti_affinity(self):
        """Tests the ServerGroupAntiAffinityFilter

        Creates two servers in an anti-affinity server group and
        asserts the servers are in the group and on different hosts.
        """
        hosts = self._create_servers_with_group('anti-affinity')
        hostnames = list(hosts.values())
        self.assertNotEqual(hostnames[0], hostnames[1],
                            'Servers are on the same host: %s' % hosts)

    @decorators.idempotent_id('9d2e924a-baf4-11e7-b856-fa163e65f5ce')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAffinityFilter"),
        'ServerGroupAffinityFilter is not available.')
    def test_create_server_with_scheduler_hint_group_affinity(self):
        """Tests the ServerGroupAffinityFilter

        Creates two servers in an affinity server group and
        asserts the servers are in the group and on same host.
        """
        hosts = self._create_servers_with_group('affinity')
        hostnames = list(hosts.values())
        self.assertEqual(hostnames[0], hostnames[1],
                         'Servers are on the different hosts: %s' % hosts)



#!/usr/bin/env python

# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import gzip
import io
import os
import re
import sys
import urllib.request as urlreq

import six
import yaml

# DEVSTACK_GATE_GRENADE is either unset if grenade is not running
# or a string describing what type of grenade run to perform.
is_grenade = os.environ.get('DEVSTACK_GATE_GRENADE') is not None
dump_all_errors = True

# As logs are made clean, remove from this set
allowed_dirty = set([
    'c-api',
    'ceilometer-acentral',
    'ceilometer-acompute',
    'ceilometer-alarm-evaluator',
    'ceilometer-anotification',
    'ceilometer-api',
    'ceilometer-collector',
    'c-vol',
    'g-api',
    'h-api',
    'h-eng',
    'ir-cond',
    'n-api',
    'n-cpu',
    'n-net',
    'q-agt',
    'q-dhcp',
    'q-lbaas',
    'q-meta',
    'q-metering',
    'q-svc',
    's-proxy'])


def process_files(file_specs, url_specs, allow_lists):
    regexp = re.compile(r"^.* (ERROR|CRITICAL|TRACE) .*\[.*\-.*\]")
    logs_with_errors = []
    for (name, filename) in file_specs:
        allow_list = allow_lists.get(name, [])
        with open(filename) as content:
            if scan_content(content, regexp, allow_list):
                logs_with_errors.append(name)
    for (name, url) in url_specs:
        allow_list = allow_lists.get(name, [])
        req = urlreq.Request(url)
        req.add_header('Accept-Encoding', 'gzip')
        page = urlreq.urlopen(req)
        buf = io.StringIO(page.read())
        f = gzip.GzipFile(fileobj=buf)
        if scan_content(f.read().splitlines(), regexp, allow_list):
            logs_with_errors.append(name)
    return logs_with_errors


def scan_content(content, regexp, allow_list):
    had_errors = False
    for line in content:
        if not line.startswith("Stderr:") and regexp.match(line):
            allowed = False
            for w in allow_list:
                pat = ".*%s.*%s.*" % (w['module'].replace('.', '\\.'),
                                      w['message'])
                if re.match(pat, line):
                    allowed = True
                    break
            if not allowed or dump_all_errors:
                if not allowed:
                    had_errors = True
    return had_errors


def collect_url_logs(url):
    page = urlreq.urlopen(url)
    content = page.read()
    logs = re.findall(r'(screen-[\w-]+\.txt\.gz)</a>', content)
    return logs


def main(opts):
    if opts.directory and opts.url or not (opts.directory or opts.url):
        print("Must provide exactly one of -d or -u")
        return 1
    print("Checking logs...")
    ALLOW_LIST_FILE = os.path.join(
        os.path.abspath(os.path.dirname(os.path.dirname(__file__))),
        "etc", "allow-list.yaml")

    file_matcher = re.compile(r".*screen-([\w-]+)\.log")
    files = []
    if opts.directory:
        d = opts.directory
        for f in os.listdir(d):
            files.append(os.path.join(d, f))
    files_to_process = []
    for f in files:
        m = file_matcher.match(f)
        if m:
            files_to_process.append((m.group(1), f))

    url_matcher = re.compile(r".*screen-([\w-]+)\.txt\.gz")
    urls = []
    if opts.url:
        for logfile in collect_url_logs(opts.url):
            urls.append("%s/%s" % (opts.url, logfile))
    urls_to_process = []
    for u in urls:
        m = url_matcher.match(u)
        if m:
            urls_to_process.append((m.group(1), u))

    allow_lists = {}
    with open(ALLOW_LIST_FILE) as stream:
        loaded = yaml.safe_load(stream)
        if loaded:
            for (name, l) in six.iteritems(loaded):
                for w in l:
                    assert 'module' in w, 'no module in %s' % name
                    assert 'message' in w, 'no message in %s' % name
            allow_lists = loaded
    logs_with_errors = process_files(files_to_process, urls_to_process,
                                     allow_lists)

    failed = False
    if logs_with_errors:
        log_files = set(logs_with_errors)
        for log in log_files:
            msg = '%s log file has errors' % log
            if log not in allowed_dirty:
                msg += ' and is not allowed to have them'
                failed = True
            print(msg)
        print("\nPlease check the respective log files to see the errors")
    if failed:
        if is_grenade:
            print("Currently not failing grenade runs with errors")
            return 0
        return 1
    print("ok")
    return 0


usage = """
Find non-allow-listed log errors in log files from a devstack-gate run.
Log files will be searched for ERROR or CRITICAL messages. If any
error messages do not match any of the allow-list entries contained in
etc/allow-list.yaml, those messages will be printed to the console and
failure will be returned. A file directory containing logs or a url to the
log files of an OpenStack gate job can be provided.

The allow-list yaml looks like:

log-name:
    - module: "a.b.c"
      message: "regexp"
    - module: "a.b.c"
      message: "regexp"

repeated for each log file with an allow-list.
"""

parser = argparse.ArgumentParser(description=usage)
parser.add_argument('-d', '--directory',
                    help="Directory containing log files")
parser.add_argument('-u', '--url',
                    help="url containing logs from an OpenStack gate job")

if __name__ == "__main__":
    try:
        sys.exit(main(parser.parse_args()))
    except Exception as e:
        print("Failure in script: %s" % e)
        # Don't fail if there is a problem with the script.
        sys.exit(0)



# Copyright 2015 Cloudwatt
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.common import identity
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class QuotasNegativeTest(base.BaseAdminNetworkTest):
    """Tests the following operations in the Neutron API:

        set network quota and exceed this quota

    v2.0 of the API is assumed.
    It is also assumed that the per-project quota extension API is configured
    in /etc/neutron/neutron.conf as follows:

        quota_driver = neutron.db.quota.driver.DbQuotaDriver
    """

    @classmethod
    def skip_checks(cls):
        super(QuotasNegativeTest, cls).skip_checks()
        if not utils.is_extension_enabled('quotas', 'network'):
            msg = "quotas extension not enabled."
            raise cls.skipException(msg)

    def setUp(self):
        super(QuotasNegativeTest, self).setUp()
        name = data_utils.rand_name('test_project_')
        description = data_utils.rand_name('desc_')
        self.project = identity.identity_utils(self.os_admin).create_project(
            name=name, description=description)
        self.addCleanup(identity.identity_utils(self.os_admin).delete_project,
                        self.project['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('644f4e1b-1bf9-4af0-9fd8-eb56ac0f51cf')
    def test_network_quota_exceeding(self):
        """Test creating network when exceeding network quota will fail"""
        # Set the network quota to two
        self.admin_quotas_client.update_quotas(self.project['id'], network=2)

        # Create two networks
        n1 = self.admin_networks_client.create_network(
            project_id=self.project['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_networks_client.delete_network,
                        n1['network']['id'])
        n2 = self.admin_networks_client.create_network(
            project_id=self.project['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_networks_client.delete_network,
                        n2['network']['id'])

        # Try to create a third network while the quota is two
        with self.assertRaisesRegex(
                lib_exc.Conflict,
                r"Quota exceeded for resources: \['network'\].*"):
            n3 = self.admin_networks_client.create_network(
                project_id=self.project['id'])
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.admin_networks_client.delete_network,
                            n3['network']['id'])



# Copyright 2014 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib import decorators


class ExtensionTestJSON(base.BaseIdentityV2Test):
    """Test extensions in identity v2 API"""

    @decorators.idempotent_id('85f3f661-f54c-4d48-b563-72ae952b9383')
    def test_list_extensions(self):
        """List all the identity extensions via v2 API"""
        body = self.non_admin_client.list_extensions()['extensions']['values']
        self.assertNotEmpty(body)
        keys = ['name', 'updated', 'alias', 'links',
                'namespace', 'description']
        for value in body:
            for key in keys:
                self.assertIn(key, value)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.identity.v3 import oauth_consumers_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestOAUTHConsumerClient(base.BaseServiceTest):
    FAKE_CREATE_CONSUMER = {
        "consumer": {
            'description': 'A fake description 1'
        }

    }

    FAKE_CONSUMER_INFO = {
        "consumer": {
            'id': '6392c7d3b7a2062e09a07aa377',
            'links': {
                'self': 'http://example.com/identity/v3/' +
                        'OS-OAUTH1/consumers/g6f2l9'
            },
            'description': 'A description that is fake'
        }

    }

    FAKE_LIST_CONSUMERS = {
        'links': {
            'self': 'http://example.com/identity/v3/OS-OAUTH1/consumers/',
            'next': None,
            'previous': None
        },
        'consumers': [
            {
                'id': '6392c7d3b7a2062e09a07aa377',
                'links': {
                    'self': 'http://example.com/identity/v3/' +
                            'OS-OAUTH1/consumers/6b9f2g5'
                },
                'description': 'A description that is fake'
            },
            {
                'id': '677a855c9e3eb3a3954b36aca6',
                'links': {
                    'self': 'http://example.com/identity/v3/' +
                            'OS-OAUTH1/consumers/6a9f2366'
                },
                'description': 'A very fake description 2'
            },
            {
                'id': '9d3ac57b08d65e07826b5e506',
                'links': {
                    'self': 'http://example.com/identity/v3/' +
                            'OS-OAUTH1/consumers/626b5e506'
                },
                'description': 'A very fake description 3'
            },
            {
                'id': 'b522d163b1a18e928aca9y426',
                'links': {
                    'self': 'http://example.com/identity/v3/' +
                            'OS-OAUTH1/consumers/g7ca9426'
                },
                'description': 'A very fake description 4'
            },
            {
                'id': 'b7e47321b5ef9051f93c2049e',
                'links': {
                    'self': 'http://example.com/identity/v3/' +
                            'OS-OAUTH1/consumers/23d82049e'
                },
                'description': 'A very fake description 5'
            }
        ]
    }

    def setUp(self):
        super(TestOAUTHConsumerClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = oauth_consumers_client.OAUTHConsumerClient(fake_auth,
                                                                 'identity',
                                                                 'regionOne')

    def _test_create_consumer(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_consumer,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_CONSUMER,
            bytes_body,
            description=self.FAKE_CREATE_CONSUMER["consumer"]["description"],
            status=201)

    def _test_show_consumer(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_consumer,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CONSUMER_INFO,
            bytes_body,
            consumer_id="6392c7d3b7a2062e09a07aa377")

    def _test_list_consumers(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_consumers,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_CONSUMERS,
            bytes_body)

    def _test_update_consumer(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_consumer,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_CONSUMER_INFO,
            bytes_body,
            consumer_id="6392c7d3b7a2062e09a07aa377")

    def test_create_consumer_with_str_body(self):
        self._test_create_consumer()

    def test_create_consumer_with_bytes_body(self):
        self._test_create_consumer(bytes_body=True)

    def test_show_consumer_with_str_body(self):
        self._test_show_consumer()

    def test_show_consumer_with_bytes_body(self):
        self._test_show_consumer(bytes_body=True)

    def test_list_consumers_with_str_body(self):
        self._test_list_consumers()

    def test_list_consumers_with_bytes_body(self):
        self._test_list_consumers(bytes_body=True)

    def test_update_consumer_with_str_body(self):
        self._test_update_consumer()

    def test_update_consumer_with_bytes_body(self):
        self._test_update_consumer(bytes_body=True)

    def test_delete_consumer(self):
        self.check_service_client_function(
            self.client.delete_consumer,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            consumer_id="6392c7d3b7a2062e09a07aa377",
            status=204)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import services as schema
from tempest.lib.common import rest_client


class ServicesClient(rest_client.RestClient):
    """Client class to send CRUD Volume Services API requests"""

    def list_services(self, **params):
        """List all Cinder services.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#list-all-cinder-services
        """
        url = 'os-services'
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_services, resp, body)
        return rest_client.ResponseBody(resp, body)

    def enable_service(self, **kwargs):
        """Enable service on a host.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#enable-a-cinder-service
        """
        put_body = json.dumps(kwargs)
        resp, body = self.put('os-services/enable', put_body)
        body = json.loads(body)
        self.validate_response(schema.enable_service, resp, body)
        return rest_client.ResponseBody(resp, body)

    def disable_service(self, **kwargs):
        """Disable service on a host.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#disable-a-cinder-service
        """
        put_body = json.dumps(kwargs)
        resp, body = self.put('os-services/disable', put_body)
        body = json.loads(body)
        self.validate_response(schema.disable_service, resp, body)
        return rest_client.ResponseBody(resp, body)

    def disable_log_reason(self, **kwargs):
        """Disable scheduling for a volume service and log disabled reason.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#log-disabled-cinder-service-information
        """
        put_body = json.dumps(kwargs)
        resp, body = self.put('os-services/disable-log-reason', put_body)
        body = json.loads(body)
        self.validate_response(schema.disable_log_reason, resp, body)
        return rest_client.ResponseBody(resp, body)

    def freeze_host(self, **kwargs):
        """Freeze a Cinder backend host.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#freeze-a-cinder-backend-host
        """
        put_body = json.dumps(kwargs)
        resp, body = self.put('os-services/freeze', put_body)
        self.validate_response(schema.freeze_host, resp, body)
        return rest_client.ResponseBody(resp)

    def thaw_host(self, **kwargs):
        """Thaw a Cinder backend host.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#thaw-a-cinder-backend-host
        """
        put_body = json.dumps(kwargs)
        resp, body = self.put('os-services/thaw', put_body)
        self.validate_response(schema.thaw_host, resp, body)
        return rest_client.ResponseBody(resp)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils


class BaseSecGroupTest(base.BaseNetworkTest):

    def _create_security_group(self):
        # Create a security group
        name = data_utils.rand_name('secgroup-')
        group_create_body = (
            self.security_groups_client.create_security_group(name=name))
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self._delete_security_group,
                        group_create_body['security_group']['id'])
        self.assertEqual(group_create_body['security_group']['name'], name)
        return group_create_body, name

    def _delete_security_group(self, secgroup_id):
        self.security_groups_client.delete_security_group(secgroup_id)
        # Asserting that the security group is not found in the list
        # after deletion
        list_body = self.security_groups_client.list_security_groups()
        secgroup_list = list()
        for secgroup in list_body['security_groups']:
            secgroup_list.append(secgroup['id'])
        self.assertNotIn(secgroup_id, secgroup_list)

    def _delete_security_group_rule(self, rule_id):
        self.security_group_rules_client.delete_security_group_rule(rule_id)
        # Asserting that the security group is not found in the list
        # after deletion
        list_body = (
            self.security_group_rules_client.list_security_group_rules())
        rules_list = list()
        for rule in list_body['security_group_rules']:
            rules_list.append(rule['id'])
        self.assertNotIn(rule_id, rules_list)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import fixtures
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import images_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestImagesClient(base.BaseServiceTest):
    # Data Dictionaries used for testing #
    FAKE_IMAGE_METADATA = {
        "list":
            {"metadata": {
             "auto_disk_config": "True",
             "Label": "Changed"
             }},
        "set_item":
            {"meta": {
             "auto_disk_config": "True"
             }},
        "show_item":
            {"meta": {
             "kernel_id": "nokernel",
             }},
        "update":
            {"metadata": {
             "kernel_id": "False",
             "Label": "UpdatedImage"
             }},
        "set":
            {"metadata": {
             "Label": "Changed",
             "auto_disk_config": "True"
             }},
        "delete_item": {}
        }

    FAKE_IMAGE_DATA = {
        "list":
            {"images": [
             {"id": "70a599e0-31e7-49b7-b260-868f441e862b",
              "links": [
                    {"href": "http://openstack.example.com/v2/openstack" +
                             "/images/70a599e0-31e7-49b7-b260-868f441e862b",
                     "rel": "self"
                     }
              ],
              "name": "fakeimage7"
              }]},
        "show": {"image": {
            "created": "2011-01-01T01:02:03Z",
            "id": "70a599e0-31e7-49b7-b260-868f441e862b",
            "links": [
                {
                    "href": "http://openstack.example.com/v2/openstack" +
                            "/images/70a599e0-31e7-49b7-b260-868f441e862b",
                    "rel": "self"
                },
            ],
            "metadata": {
                "architecture": "x86_64",
                "auto_disk_config": "True",
                "kernel_id": "nokernel",
                "ramdisk_id": "nokernel"
            },
            "minDisk": 0,
            "minRam": 0,
            "name": "fakeimage7",
            "progress": 100,
            "status": "ACTIVE",
            "updated": "2011-01-01T01:02:03Z"}},
        "create": {},
        "delete": {}
        }
    func2mock = {
        'get': 'tempest.lib.common.rest_client.RestClient.get',
        'post': 'tempest.lib.common.rest_client.RestClient.post',
        'put': 'tempest.lib.common.rest_client.RestClient.put',
        'delete': 'tempest.lib.common.rest_client.RestClient.delete'}
    # Variable definition
    FAKE_IMAGE_ID = FAKE_IMAGE_DATA['show']['image']['id']
    FAKE_SERVER_ID = "80a599e0-31e7-49b7-b260-868f441e343f"
    FAKE_CREATE_INFO = {'location': 'None'}
    FAKE_METADATA = FAKE_IMAGE_METADATA['show_item']['meta']

    def setUp(self):
        super(TestImagesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = images_client.ImagesClient(fake_auth,
                                                 "compute", "regionOne")

    def _test_image_operation(self, operation="delete", bytes_body=False):
        response_code = 200
        mock_operation = self.func2mock['get']
        expected_op = self.FAKE_IMAGE_DATA[operation]
        params = {"image_id": self.FAKE_IMAGE_ID}
        headers = None
        if operation == 'list':
            function = self.client.list_images
        elif operation == 'show':
            function = self.client.show_image
        elif operation == 'create':
            function = self.client.create_image
            mock_operation = self.func2mock['post']
            params = {"server_id": self.FAKE_SERVER_ID}
            response_code = 202
            headers = {
                'connection': 'keep-alive',
                'content-length': '0',
                'content-type': 'application/json',
                'status': '202',
                'x-compute-request-id': 'req-fake',
                'vary': 'accept-encoding',
                'x-openstack-nova-api-version': 'v2.1',
                'date': '13 Oct 2015 05:55:36 GMT',
                'location': 'http://fake.com/images/fake'
            }
        else:
            function = self.client.delete_image
            mock_operation = self.func2mock['delete']
            response_code = 204

        self.check_service_client_function(
            function, mock_operation, expected_op,
            bytes_body, response_code, headers, **params)

    def _test_image_metadata(self, operation="set_item", bytes_body=False):
        response_code = 200
        expected_op = self.FAKE_IMAGE_METADATA[operation]
        if operation == 'list':
            function = self.client.list_image_metadata
            mock_operation = self.func2mock['get']
            params = {"image_id": self.FAKE_IMAGE_ID}

        elif operation == 'set':
            function = self.client.set_image_metadata
            mock_operation = self.func2mock['put']
            params = {"image_id": "_dummy_data",
                      "meta": self.FAKE_METADATA}

        elif operation == 'update':
            function = self.client.update_image_metadata
            mock_operation = self.func2mock['post']
            params = {"image_id": self.FAKE_IMAGE_ID,
                      "meta": self.FAKE_METADATA}

        elif operation == 'show_item':
            mock_operation = self.func2mock['get']
            function = self.client.show_image_metadata_item
            params = {"image_id": self.FAKE_IMAGE_ID,
                      "key": "123"}

        elif operation == 'delete_item':
            function = self.client.delete_image_metadata_item
            mock_operation = self.func2mock['delete']
            response_code = 204
            params = {"image_id": self.FAKE_IMAGE_ID,
                      "key": "123"}

        else:
            function = self.client.set_image_metadata_item
            mock_operation = self.func2mock['put']
            params = {"image_id": self.FAKE_IMAGE_ID,
                      "key": "123",
                      "meta": self.FAKE_METADATA}

        self.check_service_client_function(
            function, mock_operation, expected_op,
            bytes_body, response_code, **params)

    def _test_resource_deleted(self, bytes_body=False):
        params = {"id": self.FAKE_IMAGE_ID}
        expected_op = self.FAKE_IMAGE_DATA['show']
        self.useFixture(
            fixtures.MockPatch(
                'tempest.lib.services.compute'
                '.images_client.ImagesClient.show_image',
                side_effect=lib_exc.NotFound))
        self.assertEqual(True, self.client.is_resource_deleted(**params))
        tempdata = copy.deepcopy(self.FAKE_IMAGE_DATA['show'])
        tempdata['image']['id'] = None
        self.useFixture(
            fixtures.MockPatch(
                'tempest.lib.services.compute'
                '.images_client.ImagesClient.show_image',
                return_value=expected_op))
        self.assertEqual(False, self.client.is_resource_deleted(**params))

    def test_list_images_with_str_body(self):
        self._test_image_operation('list')

    def test_list_images_with_bytes_body(self):
        self._test_image_operation('list', True)

    def test_show_image_with_str_body(self):
        self._test_image_operation('show')

    def test_show_image_with_bytes_body(self):
        self._test_image_operation('show', True)

    def test_create_image_with_str_body(self):
        self._test_image_operation('create')

    def test_create_image_with_bytes_body(self):
        self._test_image_operation('create', True)

    def test_delete_image_with_str_body(self):
        self._test_image_operation('delete')

    def test_delete_image_with_bytes_body(self):
        self._test_image_operation('delete', True)

    def test_list_image_metadata_with_str_body(self):
        self._test_image_metadata('list')

    def test_list_image_metadata_with_bytes_body(self):
        self._test_image_metadata('list', True)

    def test_set_image_metadata_with_str_body(self):
        self._test_image_metadata('set')

    def test_set_image_metadata_with_bytes_body(self):
        self._test_image_metadata('set', True)

    def test_update_image_metadata_with_str_body(self):
        self._test_image_metadata('update')

    def test_update_image_metadata_with_bytes_body(self):
        self._test_image_metadata('update', True)

    def test_set_image_metadata_item_with_str_body(self):
        self._test_image_metadata()

    def test_set_image_metadata_item_with_bytes_body(self):
        self._test_image_metadata(bytes_body=True)

    def test_show_image_metadata_item_with_str_body(self):
        self._test_image_metadata('show_item')

    def test_show_image_metadata_item_with_bytes_body(self):
        self._test_image_metadata('show_item', True)

    def test_delete_image_metadata_item_with_str_body(self):
        self._test_image_metadata('delete_item')

    def test_delete_image_metadata_item_with_bytes_body(self):
        self._test_image_metadata('delete_item', True)

    def test_resource_delete_with_str_body(self):
        self._test_resource_deleted()

    def test_resource_delete_with_bytes_body(self):
        self._test_resource_deleted(True)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from oslo_log import log

from tempest.api.compute import base
from tempest.common import compute
from tempest.common import utils
from tempest.common.utils import net_utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils.linux import remote_client
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF

LOG = log.getLogger(__name__)


class AttachInterfacesTestBase(base.BaseV2ComputeTest):

    @classmethod
    def skip_checks(cls):
        super(AttachInterfacesTestBase, cls).skip_checks()
        if not CONF.service_available.neutron:
            raise cls.skipException("Neutron is required")
        if not CONF.compute_feature_enabled.interface_attach:
            raise cls.skipException("Interface attachment is not available.")
        if not CONF.validation.run_validation:
            raise cls.skipException('Validation should be enabled to ensure '
                                    'guest OS is running and capable of '
                                    'processing ACPI events.')

    @classmethod
    def setup_credentials(cls):
        # This test class requires network and subnet
        cls.set_network_resources(network=True, subnet=True, router=True,
                                  dhcp=True)
        super(AttachInterfacesTestBase, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(AttachInterfacesTestBase, cls).setup_clients()
        cls.subnets_client = cls.os_primary.subnets_client
        cls.ports_client = cls.os_primary.ports_client

    def _wait_for_validation(self, server, validation_resources):
        linux_client = remote_client.RemoteClient(
            self.get_server_ip(server, validation_resources),
            self.image_ssh_user,
            self.image_ssh_password,
            validation_resources['keypair']['private_key'],
            server=server,
            servers_client=self.servers_client)
        linux_client.validate_authentication()

    def _create_server_get_interfaces(self):
        validation_resources = self.get_test_validation_resources(
            self.os_primary)
        server = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until='ACTIVE')
        # NOTE(mgoddard): Get detailed server to ensure addresses are present
        # in fixed IP case.
        server = self.servers_client.show_server(server['id'])['server']
        # NOTE(artom) self.create_test_server adds cleanups, but this is
        # apparently not enough? Add cleanup here.
        self.addCleanup(self.delete_server, server['id'])
        self._wait_for_validation(server, validation_resources)
        try:
            fip = set([validation_resources['floating_ip']['ip']])
        except KeyError:
            fip = ()
        ifs = (self.interfaces_client.list_interfaces(server['id'])
               ['interfaceAttachments'])
        body = waiters.wait_for_interface_status(
            self.interfaces_client, server['id'], ifs[0]['port_id'], 'ACTIVE')
        ifs[0]['port_state'] = body['port_state']
        return server, ifs, fip


class AttachInterfacesTestJSON(AttachInterfacesTestBase):
    """Test attaching interfaces"""

    def wait_for_port_detach(self, port_id):
        """Waits for the port's device_id to be unset.

        :param port_id: The id of the port being detached.
        :returns: The final port dict from the show_port response.
        """
        port = self.ports_client.show_port(port_id)['port']
        device_id = port['device_id']
        start = int(time.time())

        # NOTE(mriedem): Nova updates the port's device_id to '' rather than
        # None, but it's not contractual so handle Falsey either way.
        while device_id:
            time.sleep(self.build_interval)
            port = self.ports_client.show_port(port_id)['port']
            device_id = port['device_id']

            timed_out = int(time.time()) - start >= self.build_timeout

            if device_id and timed_out:
                message = ('Port %s failed to detach (device_id %s) within '
                           'the required time (%s s).' %
                           (port_id, device_id, self.build_timeout))
                raise lib_exc.TimeoutException(message)

        return port

    def _check_interface(self, iface, server_id=None, port_id=None,
                         network_id=None, fixed_ip=None, mac_addr=None):
        if server_id:
            iface = waiters.wait_for_interface_status(
                self.interfaces_client, server_id, iface['port_id'], 'ACTIVE')
        if port_id:
            self.assertEqual(iface['port_id'], port_id)
        if network_id:
            self.assertEqual(iface['net_id'], network_id)
        if fixed_ip:
            self.assertEqual(iface['fixed_ips'][0]['ip_address'], fixed_ip)
        if mac_addr:
            self.assertEqual(iface['mac_addr'], mac_addr)

    def _test_create_interface(self, server):
        iface = (self.interfaces_client.create_interface(server['id'])
                 ['interfaceAttachment'])
        iface = waiters.wait_for_interface_status(
            self.interfaces_client, server['id'], iface['port_id'], 'ACTIVE')
        return iface

    def _test_create_interface_by_network_id(self, server, ifs):
        network_id = ifs[0]['net_id']
        iface = self.interfaces_client.create_interface(
            server['id'], net_id=network_id)['interfaceAttachment']
        self._check_interface(iface, server_id=server['id'],
                              network_id=network_id)
        return iface

    def _test_create_interface_by_port_id(self, server, ifs):
        network_id = ifs[0]['net_id']
        port = self.ports_client.create_port(
            network_id=network_id,
            name=data_utils.rand_name(self.__class__.__name__))
        port_id = port['port']['id']
        self.addCleanup(self.ports_client.delete_port, port_id)
        iface = self.interfaces_client.create_interface(
            server['id'], port_id=port_id)['interfaceAttachment']
        self._check_interface(iface, server_id=server['id'], port_id=port_id,
                              network_id=network_id)
        return iface

    def _test_create_interface_by_fixed_ips(self, server, ifs):
        network_id = ifs[0]['net_id']
        subnet_id = ifs[0]['fixed_ips'][0]['subnet_id']
        ip_list = net_utils.get_unused_ip_addresses(self.ports_client,
                                                    self.subnets_client,
                                                    network_id,
                                                    subnet_id,
                                                    1)

        fixed_ips = [{'ip_address': ip_list[0]}]
        iface = self.interfaces_client.create_interface(
            server['id'], net_id=network_id,
            fixed_ips=fixed_ips)['interfaceAttachment']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port,
                        iface['port_id'])
        self._check_interface(iface, server_id=server['id'],
                              fixed_ip=ip_list[0])
        return iface

    def _test_show_interface(self, server, ifs):
        iface = ifs[0]
        _iface = self.interfaces_client.show_interface(
            server['id'], iface['port_id'])['interfaceAttachment']
        self._check_interface(iface, port_id=_iface['port_id'],
                              network_id=_iface['net_id'],
                              fixed_ip=_iface['fixed_ips'][0]['ip_address'],
                              mac_addr=_iface['mac_addr'])

    def _test_delete_interface(self, server, ifs):
        # NOTE(danms): delete not the first or last, but one in the middle
        iface = ifs[1]
        self.interfaces_client.delete_interface(server['id'], iface['port_id'])
        _ifs = (self.interfaces_client.list_interfaces(server['id'])
                ['interfaceAttachments'])
        start = int(time.time())

        while len(ifs) == len(_ifs):
            time.sleep(self.build_interval)
            _ifs = (self.interfaces_client.list_interfaces(server['id'])
                    ['interfaceAttachments'])
            timed_out = int(time.time()) - start >= self.build_timeout
            if len(ifs) == len(_ifs) and timed_out:
                message = ('Failed to delete interface within '
                           'the required time: %s sec.' % self.build_timeout)
                raise lib_exc.TimeoutException(message)

        self.assertNotIn(iface['port_id'], [i['port_id'] for i in _ifs])
        return _ifs

    def _compare_iface_list(self, list1, list2):
        # NOTE(danms): port_state will likely have changed, so just
        # confirm the port_ids are the same at least
        list1 = [x['port_id'] for x in list1]
        list2 = [x['port_id'] for x in list2]

        self.assertEqual(sorted(list1), sorted(list2))

    @decorators.idempotent_id('73fe8f02-590d-4bf1-b184-e9ca81065051')
    @utils.services('network')
    def test_create_list_show_delete_interfaces_by_network_port(self):
        """Test create/list/show/delete interfaces by network port"""
        server, ifs, _ = self._create_server_get_interfaces()
        interface_count = len(ifs)
        self.assertGreater(interface_count, 0)

        try:
            iface = self._test_create_interface(server)
        except lib_exc.BadRequest as e:
            msg = ('Multiple possible networks found, use a Network ID to be '
                   'more specific.')
            if not CONF.compute.fixed_network_name and str(e) == msg:
                raise
        else:
            ifs.append(iface)

        iface = self._test_create_interface_by_network_id(server, ifs)
        ifs.append(iface)

        iface = self._test_create_interface_by_port_id(server, ifs)
        ifs.append(iface)

        _ifs = (self.interfaces_client.list_interfaces(server['id'])
                ['interfaceAttachments'])
        self._compare_iface_list(ifs, _ifs)

        self._test_show_interface(server, ifs)

        _ifs = self._test_delete_interface(server, ifs)
        self.assertEqual(len(ifs) - 1, len(_ifs))

    @decorators.idempotent_id('d290c06c-f5b3-11e7-8ec8-002293781009')
    @utils.services('network')
    def test_create_list_show_delete_interfaces_by_fixed_ip(self):
        """Test create/list/show/delete interfaces by fixed ip"""
        # NOTE(zhufl) By default only project that is admin or network owner
        # or project with role advsvc is authorised to create interfaces with
        # fixed-ip, so if we don't create network for each project, do not
        # test _test_create_interface_by_fixed_ips.
        if not (CONF.auth.use_dynamic_credentials and
                CONF.auth.create_isolated_networks and
                not CONF.network.shared_physical_network):
            raise self.skipException("Only owner network supports "
                                     "creating interface by fixed ip.")

        server, ifs, _ = self._create_server_get_interfaces()
        interface_count = len(ifs)
        self.assertGreater(interface_count, 0)

        iface = self._test_create_interface_by_fixed_ips(server, ifs)
        ifs.append(iface)

        _ifs = (self.interfaces_client.list_interfaces(server['id'])
                ['interfaceAttachments'])
        self._compare_iface_list(ifs, _ifs)

        self._test_show_interface(server, ifs)

        _ifs = self._test_delete_interface(server, ifs)
        self.assertEqual(len(ifs) - 1, len(_ifs))

    @decorators.idempotent_id('2f3a0127-95c7-4977-92d2-bc5aec602fb4')
    def test_reassign_port_between_servers(self):
        """Tests reassigning port between servers

        1. Create a port in Neutron.
        2. Create two servers in Nova.
        3. Attach the port to the first server.
        4. Detach the port from the first server.
        5. Attach the port to the second server.
        6. Detach the port from the second server.
        """
        network = self.get_tenant_network()
        network_id = network['id']
        port = self.ports_client.create_port(
            network_id=network_id,
            name=data_utils.rand_name(self.__class__.__name__))
        port_id = port['port']['id']
        self.addCleanup(self.ports_client.delete_port, port_id)

        # NOTE(artom) We create two servers one at a time because
        # create_test_server doesn't support multiple validatable servers.
        validation_resources = self.get_test_validation_resources(
            self.os_primary)

        def _create_validatable_server():
            _, servers = compute.create_test_server(
                self.os_primary, tenant_network=network,
                wait_until='ACTIVE', validatable=True,
                validation_resources=validation_resources)
            return servers[0]

        servers = [_create_validatable_server(), _create_validatable_server()]

        # add our cleanups for the servers since we bypassed the base class
        for server in servers:
            self.addCleanup(self.delete_server, server['id'])

        for server in servers:
            # NOTE(mgoddard): Get detailed server to ensure addresses are
            # present in fixed IP case.
            server = self.servers_client.show_server(server['id'])['server']
            self._wait_for_validation(server, validation_resources)
            # attach the port to the server
            iface = self.interfaces_client.create_interface(
                server['id'], port_id=port_id)['interfaceAttachment']
            self._check_interface(iface, server_id=server['id'],
                                  port_id=port_id)

            # detach the port from the server; this is a cast in the compute
            # API so we have to poll the port until the device_id is unset.
            self.interfaces_client.delete_interface(server['id'], port_id)
            self.wait_for_port_detach(port_id)


class AttachInterfacesUnderV243Test(AttachInterfacesTestBase):
    """Test attaching interfaces with compute microversion less than 2.44"""

    max_microversion = '2.43'

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('c7e0e60b-ee45-43d0-abeb-8596fd42a2f9')
    @utils.services('network')
    def test_add_remove_fixed_ip(self):
        """Test adding and removing fixed ip from server"""
        # NOTE(zhufl) By default only project that is admin or network owner
        # or project with role advsvc is authorised to add interfaces with
        # fixed-ip, so if we don't create network for each project, do not
        # test
        if not (CONF.auth.use_dynamic_credentials and
                CONF.auth.create_isolated_networks and
                not CONF.network.shared_physical_network):
            raise self.skipException("Only owner network supports "
                                     "creating interface by fixed ip.")
        # Add and Remove the fixed IP to server.
        server, ifs, fip = self._create_server_get_interfaces()
        original_interface_count = len(ifs)  # This is the number of ports.
        self.assertGreater(original_interface_count, 0)
        # Get the starting list of IPs on the server.
        addresses = self.os_primary.servers_client.list_addresses(
            server['id'])['addresses']
        # There should be one entry for the single network mapped to a list of
        # addresses, which at this point should have at least one entry.
        # Note that we could start with two addresses depending on how tempest
        # is configured for using floating IPs.
        self.assertEqual(1, len(addresses), addresses)  # number of networks
        # Keep track of the original addresses so we can know which IP is new.
        original_ips = [addr['addr'] for addr in list(addresses.values())[0]]
        # Make sure the floating IP possibly assigned during
        # server creation is always present in the set of original ips.
        original_ips = set(original_ips).union(fip)
        original_ip_count = len(original_ips)
        self.assertGreater(original_ip_count, 0, addresses)  # at least 1
        network_id = ifs[0]['net_id']
        # Add another fixed IP to the server. This should result in another
        # fixed IP on the same network (and same port since we only have one
        # port).
        self.servers_client.add_fixed_ip(server['id'], networkId=network_id)

        def _wait_for_ip_change(expected_count):
            _addresses = self.os_primary.servers_client.list_addresses(
                server['id'])['addresses']
            _ips = set([addr['addr'] for addr in list(_addresses.values())[0]])
            # Make sure possible floating ip is always present in the set.
            _ips = _ips.union(fip)
            LOG.debug("Wait for change of IPs. All IPs still associated to "
                      "the server %(id)s: %(ips)s",
                      {'id': server['id'], 'ips': _ips})
            return len(_ips) == expected_count

        # Wait for the ips count to increase by one.
        if not test_utils.call_until_true(
                _wait_for_ip_change, CONF.compute.build_timeout,
                CONF.compute.build_interval, original_ip_count + 1):
            raise lib_exc.TimeoutException(
                'Timed out while waiting for IP count to increase.')

        # Remove the fixed IP that we just added.
        server_detail = self.os_primary.servers_client.show_server(
            server['id'])['server']
        # Get the Fixed IP from server.
        fixed_ip = None
        for ip_set in server_detail['addresses']:
            for ip in server_detail['addresses'][ip_set]:
                if (ip['OS-EXT-IPS:type'] == 'fixed' and
                        ip['addr'] not in original_ips):
                    fixed_ip = ip['addr']
                    break
            if fixed_ip is not None:
                break
        self.servers_client.remove_fixed_ip(server['id'], address=fixed_ip)
        # Wait for the interface count to decrease by one.
        if not test_utils.call_until_true(
                _wait_for_ip_change, CONF.compute.build_timeout,
                CONF.compute.build_interval, original_ip_count):
            raise lib_exc.TimeoutException(
                'Timed out while waiting for IP count to decrease.')


class AttachInterfacesV270Test(AttachInterfacesTestBase):
    """Test interface API with microversion greater than 2.69"""
    min_microversion = '2.70'

    @decorators.idempotent_id('2853f095-8277-4067-92bd-9f10bd4f8e0c')
    @utils.services('network')
    def test_create_get_list_interfaces(self):
        """Test interface API with microversion greater than 2.69

        Checking create, get, list interface APIs response schema.
        """
        server = self.create_test_server(wait_until='ACTIVE')
        try:
            iface = self.interfaces_client.create_interface(server['id'])[
                'interfaceAttachment']
            iface = waiters.wait_for_interface_status(
                self.interfaces_client, server['id'], iface['port_id'],
                'ACTIVE')
        except lib_exc.BadRequest as e:
            msg = ('Multiple possible networks found, use a Network ID to be '
                   'more specific.')
            if not CONF.compute.fixed_network_name and str(e) == msg:
                raise
        else:
            # just to check the response schema
            self.interfaces_client.show_interface(
                server['id'], iface['port_id'])
            self.interfaces_client.list_interfaces(server['id'])



# Copyright 2014 Hewlett-Packard Development Company, L.P
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class RegionsTestJSON(base.BaseIdentityV3AdminTest):
    """Test regions"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @classmethod
    def setup_clients(cls):
        super(RegionsTestJSON, cls).setup_clients()
        cls.client = cls.regions_client

    @classmethod
    def resource_setup(cls):
        super(RegionsTestJSON, cls).resource_setup()
        cls.setup_regions = list()
        for _ in range(2):
            r_description = data_utils.rand_name('description')
            region = cls.client.create_region(
                description=r_description)['region']
            cls.addClassResourceCleanup(
                cls.client.delete_region, region['id'])
            cls.setup_regions.append(region)

    @decorators.idempotent_id('56186092-82e4-43f2-b954-91013218ba42')
    def test_create_update_get_delete_region(self):
        """Test creating, updating, getting and updating region"""
        # Create region
        r_description = data_utils.rand_name('description')
        region = self.client.create_region(
            description=r_description,
            parent_region_id=self.setup_regions[0]['id'])['region']
        # This test will delete the region as part of the validation
        # procedure, so it needs a different cleanup method that
        # would be useful in case the tests fails at any point before
        # reaching the deletion part.
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.client.delete_region, region['id'])
        self.assertEqual(r_description, region['description'])
        self.assertEqual(self.setup_regions[0]['id'],
                         region['parent_region_id'])
        # Update region with new description and parent ID
        r_alt_description = data_utils.rand_name('description')
        region = self.client.update_region(
            region['id'],
            description=r_alt_description,
            parent_region_id=self.setup_regions[1]['id'])['region']
        self.assertEqual(r_alt_description, region['description'])
        self.assertEqual(self.setup_regions[1]['id'],
                         region['parent_region_id'])
        # Get the details of region
        region = self.client.show_region(region['id'])['region']
        self.assertEqual(r_alt_description, region['description'])
        self.assertEqual(self.setup_regions[1]['id'],
                         region['parent_region_id'])
        # Delete the region
        self.client.delete_region(region['id'])
        body = self.client.list_regions()['regions']
        regions_list = [r['id'] for r in body]
        self.assertNotIn(region['id'], regions_list)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('2c12c5b5-efcf-4aa5-90c5-bff1ab0cdbe2')
    def test_create_region_with_specific_id(self):
        """Test creating region with specific id"""
        r_region_id = data_utils.rand_uuid()
        r_description = data_utils.rand_name('description')
        region = self.client.create_region(
            region_id=r_region_id, description=r_description)['region']
        self.addCleanup(self.client.delete_region, region['id'])
        # Asserting Create Region with specific id response body
        self.assertEqual(r_region_id, region['id'])
        self.assertEqual(r_description, region['description'])

    @decorators.idempotent_id('d180bf99-544a-445c-ad0d-0c0d27663796')
    def test_list_regions(self):
        """Test getting a list of regions"""
        fetched_regions = self.client.list_regions()['regions']
        missing_regions =\
            [e for e in self.setup_regions if e not in fetched_regions]
        # Asserting List Regions response
        self.assertEmpty(missing_regions,
                         "Failed to find region %s in fetched list" %
                         ', '.join(str(e) for e in missing_regions))

    @decorators.idempotent_id('2d1057cb-bbde-413a-acdf-e2d265284542')
    def test_list_regions_filter_by_parent_region_id(self):
        """Test listing regions filtered by parent region id"""
        # Add a sub-region to one of the existing test regions
        r_description = data_utils.rand_name('description')
        region = self.client.create_region(
            description=r_description,
            parent_region_id=self.setup_regions[0]['id'])['region']
        self.addCleanup(self.client.delete_region, region['id'])
        # Get the list of regions filtering with the parent_region_id
        params = {'parent_region_id': self.setup_regions[0]['id']}
        fetched_regions = self.client.list_regions(params=params)['regions']
        # Asserting list regions response
        self.assertIn(region, fetched_regions)
        for r in fetched_regions:
            self.assertEqual(self.setup_regions[0]['id'],
                             r['parent_region_id'])



# Copyright 2018 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

# NOTE(gmann): This file is to write the tests which mainly
# tests newly added microversion schema related to servers APIs.
# As per (https://docs.openstack.org/tempest/latest/microversion_testing.
# html#tempest-scope-for-microversion-testing),
# we need to fill the API response schema gaps which gets modified
# during microversion change. To cover the testing of such schema
# we need to have operation schema test which just test
# the microversion schemas.
# If you are adding server APIs microversion schema file without
# their integration tests, you can add tests to cover those schema
# in this file.


class ServerShowV254Test(base.BaseV2ComputeTest):
    """Test servers API schema for compute microversion greater than 2.53"""
    min_microversion = '2.54'
    max_microversion = 'latest'

    @decorators.idempotent_id('09170a98-4940-4637-add7-1a35121f1a5a')
    def test_rebuild_server(self):
        """Test rebuilding server with microversion greater than 2.53"""
        server = self.create_test_server(wait_until='ACTIVE')
        keypair_name = data_utils.rand_name(
            self.__class__.__name__ + '-keypair')
        kwargs = {'name': keypair_name}
        self.keypairs_client.create_keypair(**kwargs)
        self.addCleanup(self.keypairs_client.delete_keypair,
                        keypair_name)
        # Checking rebuild API response schema
        self.servers_client.rebuild_server(server['id'], self.image_ref_alt,
                                           key_name=keypair_name)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')


class ServerShowV257Test(base.BaseV2ComputeTest):
    """Test servers API schema for compute microversion greater than 2.56"""
    min_microversion = '2.57'
    max_microversion = 'latest'

    @decorators.idempotent_id('803df848-080a-4261-8f11-b020cd9b6f60')
    def test_rebuild_server(self):
        """Test rebuilding server with microversion greater than 2.56"""
        server = self.create_test_server(wait_until='ACTIVE')
        user_data = "ZWNobyAiaGVsbG8gd29ybGQi"
        # Checking rebuild API response schema
        self.servers_client.rebuild_server(server['id'], self.image_ref_alt,
                                           user_data=user_data)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from tempest.lib.services.network import base as base_network_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib import fake_http
from tempest.tests.lib.services import base


class TestBaseNetworkClient(base.BaseServiceTest):

    def setUp(self):
        super(TestBaseNetworkClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = base_network_client.BaseNetworkClient(
            fake_auth, 'compute', 'regionOne')

        self.mock_expected_success = mock.patch.object(
            self.client, 'expected_success').start()

    def _assert_empty(self, resp):
        self.assertEqual([], list(resp.keys()))

    @mock.patch('tempest.lib.common.rest_client.RestClient.post')
    def test_create_resource(self, mock_post):
        response = fake_http.fake_http_response(headers=None, status=201)
        mock_post.return_value = response, '{"baz": "qux"}'

        post_data = {'foo': 'bar'}
        resp = self.client.create_resource('/fake_url', post_data)

        self.assertEqual({'status': '201'}, resp.response)
        self.assertEqual("qux", resp["baz"])
        mock_post.assert_called_once_with('v2.0/fake_url', '{"foo": "bar"}')
        self.mock_expected_success.assert_called_once_with(
            201, 201)

    @mock.patch('tempest.lib.common.rest_client.RestClient.post')
    def test_create_resource_expect_different_values(self, mock_post):
        response = fake_http.fake_http_response(headers=None, status=200)
        mock_post.return_value = response, '{}'

        post_data = {'foo': 'bar'}
        resp = self.client.create_resource('/fake_url', post_data,
                                           expect_response_code=200,
                                           expect_empty_body=True)

        self.assertEqual({'status': '200'}, resp.response)
        self._assert_empty(resp)
        mock_post.assert_called_once_with('v2.0/fake_url', '{"foo": "bar"}')
        self.mock_expected_success.assert_called_once_with(
            200, 200)

    @mock.patch('tempest.lib.common.rest_client.RestClient.put')
    def test_update_resource(self, mock_put):
        response = fake_http.fake_http_response(headers=None, status=200)
        mock_put.return_value = response, '{"baz": "qux"}'

        put_data = {'foo': 'bar'}
        resp = self.client.update_resource('/fake_url', put_data)

        self.assertEqual({'status': '200'}, resp.response)
        self.assertEqual("qux", resp["baz"])
        mock_put.assert_called_once_with('v2.0/fake_url', '{"foo": "bar"}')
        self.mock_expected_success.assert_called_once_with(
            200, 200)

    @mock.patch('tempest.lib.common.rest_client.RestClient.put')
    def test_update_resource_expect_different_values(self, mock_put):
        response = fake_http.fake_http_response(headers=None, status=201)
        mock_put.return_value = response, '{}'

        put_data = {'foo': 'bar'}
        resp = self.client.update_resource('/fake_url', put_data,
                                           expect_response_code=201,
                                           expect_empty_body=True)

        self.assertEqual({'status': '201'}, resp.response)
        self._assert_empty(resp)
        mock_put.assert_called_once_with('v2.0/fake_url', '{"foo": "bar"}')
        self.mock_expected_success.assert_called_once_with(
            201, 201)
